@@ -550,12 +550,16 @@ public:
     /**
      * Append a new child of this XmlParent, release the object on failure
      * @param child The child to append
+     * @param code Optional pointr to error code to be filled on failure
      * @return The child on success, 0 on failure
      */
-    inline XmlChild* addChildSafe(XmlChild* child) {
+    inline XmlChild* addChildSafe(XmlChild* child, XmlSaxParser::Error* code = 0) {
 	    XmlSaxParser::Error err = addChild(child);
-	    if (err != XmlSaxParser::NoError)
+	    if (err != XmlSaxParser::NoError) {
 		TelEngine::destruct(child);
+		if (code)
+		    *code = err;
+	    }
 	    return child;
 	}
 
@@ -777,6 +781,13 @@ public:
      */
     virtual XmlDoctype* xmlDoctype()
 	{ return 0; }
+
+    /**
+     * Replaces all ${paramname} with the corresponding parameters
+     * @param params List of parameters
+     */
+    virtual void replaceParams(const NamedList& params)
+	{}
 };
 
 
@@ -915,6 +926,13 @@ public:
     virtual void clearChildren()
 	{ m_list.clear(); }
 
+    /**
+     * Copy other fragment into this one
+     * @param other Fragment to copy
+     * @param parent Optional parent to set in copied children
+     */
+    void copy(const XmlFragment& other, XmlParent* parent = 0);
+
     /**
      * Build a String from this XmlFragment
      * @param dump The string where to append representation
@@ -931,6 +949,12 @@ public:
 	const String& origIndent = String::empty(), bool completeOnly = true,
 	const String* auth = 0, const XmlElement* parent = 0) const;
 
+    /**
+     * Replaces all ${paramname} in fragment's children with the corresponding parameters
+     * @param params List of parameters
+     */
+    void replaceParams(const NamedList& params);
+
     /**
      * Find a completed xml element in a list
      * @param list The list to search for the element
@@ -990,6 +1014,15 @@ public:
      */
     XmlDeclaration* declaration() const;
 
+    /**
+     * Retrieve XML fragment outside the root element
+     * @param before True to retrieve the fragment holding children before root element,
+     *  false to retrieve children after root
+     * @return Requested fragment's reference
+     */
+    inline const XmlFragment& getFragment(bool before) const
+	{ return before ? m_beforeRoot : m_afterRoot; }
+
     /**
      * Retrieve the root element
      * @param completed True to retrieve the root element if is not completed
@@ -1064,10 +1097,12 @@ public:
      * @param escape True if the attributes values need to be escaped
      * @param indent Spaces for output
      * @param completeOnly True to build only if complete
+     * @param eoln End of line chars. Set it to NULL to ignore it. Empty string will be replaced by CR/LF
      * @return 0 on success, error code on failure
      */
     int saveFile(const char* file = 0, bool escape = true,
-	const String& indent = String::empty(), bool completeOnly = true) const;
+	const String& indent = String::empty(), bool completeOnly = true,
+	const char* eoln = "\r\n") const;
 
     /**
      * Build a String from this XmlDocument
@@ -1079,6 +1114,12 @@ public:
     void toString(String& dump, bool escape = true, const String& indent = String::empty(),
 	const String& origIndent = String::empty()) const;
 
+    /**
+     * Replaces all ${paramname} in document's components with the corresponding parameters
+     * @param params List of parameters
+     */
+    void replaceParams(const NamedList& params);
+
 private:
     XmlElement* m_root;                  // The root element
     XmlFragment m_beforeRoot;            // XML children before root (declaration ...)
@@ -1530,6 +1571,13 @@ public:
     bool setXmlns(const String& name = String::empty(), bool addAttr = false,
 	const String& value = String::empty());
 
+    /**
+     * Replaces all ${paramname} in element's attributes and children with the
+     *  corresponding parameters
+     * @param params List of parameters
+     */
+    virtual void replaceParams(const NamedList& params);
+
     /**
      * Check if a string represents a namespace attribute name
      * @param str The string to check
@@ -1775,6 +1823,13 @@ public:
      * @return False if the text contains non space characters.
      */
     bool onlySpaces();
+
+    /**
+     * Replaces all ${paramname} in text with the corresponding parameters
+     * @param params List of parameters
+     */
+    virtual void replaceParams(const NamedList& params);
+
 private:
     String m_text;                        // The text
 };
